;***********  Rolladen-Steuerung fr 9 Rollden  ********************
;
;  Datei 		:	Rolladen.asm
;  Autor			:	Hanno Grff
;  Version		:	1.0
;  Datum			:	Dezember 2009
;
;  Compiler    :	AVR Assembler
;
;	Anderungen  :	
;	
;	Dank			:	an Peter Danegger fr die Uhr-Routine und Tastenentprellung
;         			an Andreas Schwarz fr die LCD-Routinen
;
;  Bemerkungen :  - OnOffFlag Bit7: 1= Sommerzeit, 0= Winterzeit
;                   bei Start whrend Sommerzeit anpassen (Eeprom)
;                 - Ausschaltzeit im Eeprom derzeit bei 35 Sekunden
;
;
;********************************************************************************************; ******************************************************


.include "I:\vmlab\include\m16def.inc"

; Variablendefinition

.equ  xtal = 11059700
.equ  debounce = 256
.equ  remainder = xtal - xtal/debounce * debounce

.def	Null			 =r1
.def	IsReg	 		 =r2
.def	Key_State	 =r3
.def	Key_Press	 =r4
.def	Key_Old		 =r5

.def	OnOffFlag	 =r6 ; Bit0/1 = Sensor an/aus // Bit2/3 = Timer1 an/aus // Bit4/5 =Timer 2 an/aus, Bit7 = Sommer/Winter
.def  RollFlag		 =r7 ; 0-8 = Rollden /9 = Alle/10 u. 11 = Gruppen
.def	TimeFlag		 =r8 ; wird fr diverse Zeitnahmen (Warteschleifen) bentigt
.def  UpDnFlag		 =r9 ; Flag fr Hoch/Runter
.def  H_MenuFlag	 =r10 ; Flag Hauptmen
.def  U_MenuFlag   =r11; Flag Untermen

.def	AusschaltZeit=r12;
.def	ADCLow		 =r13; Sensor
.def	ADCHigh		 =r14; Sensor
.def	ADCTimer	    =r15; Wartezeit bis erneute Messung

.def	Prescaler 	 =r16; Uhr-Register
.def	Second 		 =r17
.def	Minute		 =r18	
.def	Hour			 =r19	
.def	Day			 =r20	
.def	Month			 =r21	
.def  Year     	 =r22

.def  AL  			 =r23; Allg. Register
.def	AH  			 =r24
.def	BL				 =r25	
.def	BH				 =r26; =XL	
.def	CL				 =r27; =XH


;PortA
.equ	Key_Pin				= PINA
.equ	Key_Port 			= PORTA

.equ	Taste1				= PINA1
.equ	Taste2				= PINA2
.equ	Taste3				= PINA3
.equ	Taste4				= PINA4

;PORTB
.equ	LCD_PORT				= PortB
.equ 	PIN_LCD_D4			= PINB0
.equ 	PIN_LCD_D5			= PINB1
.equ 	PIN_LCD_D6			= PINB2
.equ 	PIN_LCD_D7			= PINB3
.equ 	PIN_LCD_RS			= PINB5
.equ 	PIN_LCD_Enable		= PINB4

.cseg
; Interrupt-Vektoren
reset:
	   rjmp init
   	reti      ; Addr $01
   	reti      ; Addr $02
	   reti      ; Addr $03
	   reti      ; Addr $04
	   reti      ; Addr $05
	   reti      ; Addr $06
	   reti      ; Addr $07
	   reti      ; Addr $08
	   reti      ; Addr $09
	   reti      ; Addr $0A
	   reti      ; Addr $0B
	   rjmp      OC1AInt	
	   reti      ; Addr $0D
	   reti      ; Addr $0E
	   reti      ; Addr $0F
	   reti      ; Addr $10

;
;*************************************************************************************
;Timer-Interrupt-Routine alle 4 ms
;*************************************************************************************
OC1Aint:
		push	AL							
		push	AH							; Universalregister sichern
		push	BL
	
		in		IsReg,SReg

		dec	TimeFlag					; wird fr diverse Zeitnahmen bentigt

		mov	AL, Key_Old				; evtl. Tastendruck auswerten	
		in		Key_Old, Key_Pin     ; und mit Key_old vergleichen
		eor	AL, Key_Old					
		com	Key_Old
		mov	AH, Key_State
		or		Key_State,AL
		and	AL, Key_Old
		eor	Key_State,AL
		and 	AH, AL
		or		Key_Press,AH

		ldi	BL, high(xtal/debounce-1)
		out	OCR1AH,BL
		ldi	BL,low(xtal/debounce-1)
		out 	OCR1AL,BL
		dec	Prescaler
		brne	oci1
	
		ldi	Prescaler, Debounce
		inc	Second  					; zhlt bis 60
		inc	ADCTimer             ; zhlt bis 15

		tst 	Ausschaltzeit			; ist Ausschaltzeit gesetzt
		breq	oci0						; wenn nein weiter im Hauptprogramm

		dec	Ausschaltzeit			; sonst Ausschaltzeit = Ausschaltzeit-1
		tst	Ausschaltzeit			; ist Ausschaltzeit=0
		brne	oci0						; wenn nein weiter im Programm
		rcall	Rolladenaus				; sonst schalte alle Rollden stromlos

oci0:
		ldi	BL, high(xtal/debounce+remainder-1)
		out	OCR1AH, BL
		ldi	BL, low(xtal/debounce+remainder-1)
		out	OCR1AL, BL	

oci1:
		out 	SReg,IsReg
		pop	BL
		pop	AH
		pop	AL
		reti

;*************************************************************************************
;Initialisierung
;*************************************************************************************
init:
 		cli
		ldi	AL,High(RAMEND)
		out	SPH,AL
		ldi	AL, LOW(RAMEND)
		out 	SPL,AL

;****** Port-Initialisierung *********************************************************
	;Richtungsregister: 1 = Ausgang, 0 = Eingang
	;PINA0: ADC=Eingang, PINA1 bis PINA4: Tasten=Eingang, PINA6 bis PINA7:  Relais=Ausgang
		ldi	AL,0b11000000
		out	DDRA, AL
	
	;LCD=Ausgang
	;PINB7: nc, PinB6: nc, PINB5: LCD-ENABLE, PINB4: LCD-RS, PINB3: LCD-DB4, PINB2: LCD-DB5, PINB1: LCD-DB6, PINB0: LCD-DB7
		ldi 	AL, 0b00111111
		out 	DDRB, AL

	;Richtungsregister: alles Relais-Ausgnge
		ldi 	AL	,0xFF
		out 	DDRC, AL
		out 	DDRD, AL
	
   ;Pullup bei Button-Pins aktivieren
	;PortA (Taste 1-4)
		ldi	AL,(1<<PinA1)|(1<<PinA2)|(1<<PinA3)|(1<<PinA4)
		out	PortA,AL

	
;****** Variablen-Initialisierung *****************************************************

		ldi	AL,-1						; MenFlags mit -1 vorbelegen
		mov   H_MenuFlag,AL
		mov 	U_MenuFlag,AL

		clr	Null
		clr   Ausschaltzeit
		ldi	Day,1                ; Datum mit 01.01.10 vorbelegen
		ldi	Month,4
		ldi	Year,10
		ldi	Hour,12					; 12:00 Uhr
		clr	Minute
		clr	Second
		sts	dWoTag, Null			; Sonntag
		ldi	AL,9
		mov	Rollflag, AL			; Rollden = Alle

		ldi	AL,0x1E					; Tastenabfrage vorbelegen PinA1 bis PinA4
		out	Key_Port,AL
		mov	Key_Old, AL
		clr	Key_Press
      clr	Key_State
      clr	ADCHigh
      clr	ADCLow

;****** Timer-Initialisierung **********************************************************
		ldi	AL, (1<<WGM12) | (1<<CS10)
		out 	TCCR1B, AL
	
		ldi	AL, high(xtal/debounce-1)
		out	OCR1AH,AL
		ldi	AL,low(xtal/debounce-1)
		out 	OCR1AL, AL
		out	TCNT1L, AL
		ldi	prescaler, debounce
	
		ldi	AL, (1<<OCIE1A)
		out	TIMSK, AL

;****** Admux-Initialisierung **********************************************************
	; ADMUX: REFS1/REFS0/ADLAR/MUX4-MUX0)
	  	ldi	AL, 0b01000000	; AVCC und ADC0
  		out	ADMUX,AL
	; ADCSRA: ADEN,ADSC,ADATE,ADIF,ADIE,ADPS2,ADPS1,ADPS0
		ldi	AL, 0b10000111			;  Prescaler=128
		out	ADCSR,AL

;************************************************************************************
		ldi	AL,100					; erstmal 500 ms Warten,
i0:	;rcall wait5ms					; Eeprom reagiert empfindlich
      dec	AL							
      brne	i0							

;****** Sonstige Initialisierungen **************************************************
		rcall LCD_Init
		rcall LCD_LoadChars
		rcall LCD_Clear

   	rcall	ReadEeprom				; im Eeprom gespeicherte Daten ins DSEG kopieren
		rcall	PrintDatetime			; Datum und Zeit ausgeben
		rcall	PrintRolladen			; "Rolladen" und Nr. ausgeben
		rcall	PrintSensor				; Sensoranzeige
	   rcall PrintStatus				; Sensor- und Timerstatus
		rcall saveEeprom
		sei                        ; Interrupt einschalten

;*************************************************************************************
; Hauptroutine  // Uhr und Datum aufaddieren, auf Tasten warten
;*************************************************************************************
Main:
		cpi	Second,60				; sind = 60 Sekunden
		breq	Count
		
		mov	AL,ADCtimer				; alle 15 Sekunden
		cpi	AL,15						; ADC-Sensor abfragen
		brne	m1
		rcall GetADC		
		mov	AH, H_MenuFlag
		cpi	AH,-1						; wird gerade ein Men angezeigt?
		brne	m0							; dann Sensorwert nicht aktualisieren
		rcall PrintSensVal			; sonst Sensorwert anzeigen
		rcall	SensorAlarm
m0:	clr	ADCTimer					; ADCTimer wieder auf 0 setzen

m1:	mov	AL, Key_Press        ; KeyPress auswerten
		clr	Key_Press				; und zurcksetzen
		tst	AL	                  ; ist Wert = 0
		breq	Main                 ; dann zurck
	
		sbrc	AL, Taste1				; Tastelinks = Untermen whlen oder
		rcall TasteLinks           ; Rolladen whlen
	
		sbrc	AL, Taste2
		rcall	TasteRunter				; Taste2 = Menwahl runter oder Rolladen runter
	
		sbrc	AL, Taste3		      ; Taste3 = Menwahl hoch oder Rolladen hoch
		rcall TasteHoch

		sbrc	AL, Taste4				; TasteRechts = Aufruf Hauptmenu,
		rcall	TasteRechts          ; zurck und nderungen speichern

		rjmp	Main						;

; ****** Zhlroutine *******************************************************************
Count:
		ldi	second,0					; Sekunde wieder 0
		inc	Minute               ; Minute +1
		
		;rcall	getADC					; jede volle Minute ADC einlesen
		;rcall	printSensVal			; und anzeigen
	
		cpi	Minute,60            ; ist Minute= 60
		brcs	c3               		; wenn nicht weiter
		clr	Minute	            ; sonst Minute = 0
		inc	Hour                 ; Stunde +1
		cpi	Hour,24              ; Ist Stunde = 24
		brcs	c3               		; wenn nicht, weiter
		clr	Hour	               ; Stunde = 0
		
		rcall	SaveEeprom				; um 0 Uhr werden Variablen ins Eeprom kopiert

		inc	Day  						; Tag +1
		lds 	AL,dWoTag            ; Nr. des Wochentags holen
		inc	AL							; Wochentag +1
		cpi	AL,7                 ; ist es bereits der 7.?
		brne	c0                   ; wenn nicht weiter
		clr	AL		               ; sonst auf 0 setzen
c0:	sts	dWoTag,AL            ; neuen Wochentag speichern

		ldi	AL,30                ; Monat hat 30 Tage
		cpi	Month,4              ; April ?
		breq	c2               		; wenn ja weiter
		cpi	Month,6              ; Juni?
		breq	c2
		cpi	Month,9              ; September?
		breq	c2
		cpi	Month,11             ; November?
		breq	c2               		; wenn ja stimmt's mit 30 Tagen
		ldi	AL,31						; sonst 31 Tage nach AL
		cpi	Month,2              ; Februar?
		brne	c2							; wenn nicht, dann 31 Tage richtig
		ldi	AL,29                ; wenn Februar, dann Tage = 29
		sbrs	Year,0               ; Bit0 = 1- dann springen und Tage auf 28 setzen
		sbrc	Year,1               ; Bit1 = 0- dann springen und Tage auf 29 belassen
		ldi	AL,28						

c2:	cp		AL,Day					; ist Tag > AL , dann Carry gesetzt
		brcc	c3							; wenn nicht weiter bei C3
		ldi	Day,1                ; sonst Tag=1
		inc	Month						; Monat+1
		cpi	Month,13             ; ist Monat 13
		brcs	c3               		; wenn nicht weiter bei C3
		ldi	Month,1              ; Monat=1
		inc	Year                 ; Jahr +1
		cpi	Year,100             ; wenn Jahr= 100
		brcs	c3
		ldi	Year,0               ; Jahr wieder auf 0 setzen

c3:	cpi	Month,3					; Ist es Mrz?
		brne	c4
		cpi	Day,25					; wenn ja, ist Tag>=25?
		brge	c3a              		; brlo geht nicht-> zu weit
		rjmp	c5

c3a:	lds	AL,dWotag				; Wochentag aus DSEG nach AL
		tst	AL	               	; ist es auch noch Sonntag?
		brne	c5 						; wenn nicht-> Ende
		sbrc	OnOffFlag,7				; ist Bit7 gelscht dann Winterzeit und weiter prfen
	   rjmp	c5							; sonst ->Ende	
		cpi	Hour,2					; wenn ja, ist es 2 Uhr?
		brne	c5               		; wenn nicht-> Ende
		inc	Hour						; sonst von 2 Uhr auf 3 Uhr vorstellen
		
		lds	AL,dSZ1R					; eingestellte Timer-Stunden
		inc	AL							; auch
		sts	dSZ1R,AL             ; vorstellen
		lds	AL,dSZ1H
		inc	AL
		sts	dSZ1H,AL	
		lds	AL,dSZ2R
		inc	AL
		sts	dSZ2R,AL
		lds	AL,dSZ2H
		inc	AL
		sts	dSZ2H,AL
	
		ldi	AL,128					; Bit7 in AL setzen
		or		OnOffFlag,AL			; Bit7 in OnOffFlag setzen -> Sommerzeit
		rjmp	c5							; -> Ende

c4:	cpi	Month,10					; ist es Oktober?
		brne	c5							; -> Ende
		cpi	Day,25               ; ist Tag = 25
		brlo	c5							; wenn kleiner -> Ende
   	lds	AL,dWotag				
		tst	AL							; ist es auch noch Sonntag
		brne	c5               		; wenn nicht -> Ende
	   sbrs	OnOffFlag,7				; wenn Bit7 = 1 dann Sommerzeit -> weiter prfen
   	rjmp	c5							; wenn letzte Stunde schon umgestellt ->Ende
		cpi	Hour,3               ; ist es 3 Uhr
		brne	c5               		; wenn nicht -> Ende
		dec	Hour						; sonst Stunde zurcksetzen auf 2 Uhr
		lds	AL,dSZ1R					; eingestellte Timer-Stunden
		dec	AL							; auch
		sts	dSZ1R,AL             ; zurckstellen
		lds	AL,dSZ1H
		dec	AL
		sts	dSZ1H,AL	
		lds	AL,dSZ2R
		dec	AL
		sts	dSZ2R,AL
		lds	AL,dSZ2H
		dec	AL
		sts	dSZ2H,AL
		ldi	AL,128               ; Bit 7 in AL setzen
		eor	OnOffFlag,AL			; und Bit7 von OnOffFlag auf 0 setzen -> Winterzeit

c5:	mov	AH,H_MenuFlag			; HMen-Flag nach AH
		cpi	AH,-1                ; ist Hauptmen gerade aktiv?
		brne	cEnd	               ; wenn ja dann weiter
		rcall	PrintDatetime        ; sonst Datum und Uhr anzeigen
		rcall	TimerAlarm				; Timer prfen

cEnd:	rjmp	main

;*************************************************************************************
; Text-Ausgabe Hauptmenu
;*************************************************************************************
Print_H_Menu:
		mov	AH, H_MenuFlag					
		ldi	ZL,Low(Text_H_Menu*2)		; ersten Meneintrag nach Z
		ldi	ZH,High(text_H_Menu*2)
		tst	AH									; ist AH = 0?
		breq	h3									; dann Ausgabe von Hauptmen(0)
	
h2:	adiw	ZH:ZL,18							; sonst pro Zhlwert 18 Bytes addieren
		dec	AH									;
   	brne	h2								
	
h3:	rcall	LCD_Clear                  ; LCD lschen
		rcall LCD_Print						; Mentext ausgeben
		ret										; Zurck zur Hauptroutine


;*************************************************************************************
; Taste links (1):  	verteilt Tastendruck auf die Untermens
;*************************************************************************************
TasteLinks:
		push	AL
		push	AH
	
		mov	AH, H_MenuFlag    			; Hauptmen-Flag
		mov	AL, U_MenuFlag
	
;Untermen -1: Rolladenanwahl
ra:	cpi	AH,0      						; Ist Flag = -1
		brge	um0								; wenn >=0 weiter im Men
		inc	Rollflag							; sonst Rollflag erhhen
		mov	AL,Rollflag	      			; ist Rollflag bereits 12 ?
		cpi	AL,12
		brne	ra0  								; wenn nein, weiter
		clr	AL									; sonst Rollflag zurck auf 0 setzen
		clr	Rollflag

ra0:	rcall PrintRolladenNr				; Ausgabe des Rollflags = RolladenNr-1
		rjmp	umEnd								; zum Ende


; Untermen 0-1: ADC einlesen
um0:	cpi	AH,2								; ist Hauptmen = 0 oder 1?
		brge	um1	              			; wenn >= weiter bei um1
		mov	Al,U_MenuFlag					; ist U-MenuFlag
		cpi	AL,-1                      ; =-1?
		brne  um0a                       ; wenn nicht weiter
		ldi	ZH, high(dSensR)				; sonst Adressse Sensorwert runter im DSeg
		ldi	ZL, low(dSensR)
		sbrc	AH,0								; Bit0=0 -> Sensorrunter?
		adiw	ZH:ZL,3 							; sonst Adresse Sensorwert hoch im DSeg		
		ld		BH, Z+							; Wert einlesen (=ADCHigh)
		ld		BL, Z+							; Wert einlesen(=ADClow)
		ld		CL, Z								; Schaltgruppe

um0a:	ldi	AL,0x40							; 2. Zeile 1. Spalte
		rcall	lcd_locate
		inc	U_MenuFlag						; Untermen=Untermen+1
		sbrc 	U_MenuFlag,1	            ; wenn Bit1 gesetzt->MenuFlag=2
		clr	U_MenuFlag						; zurck auf 0 setzen
		
		tst	U_MenuFlag						; bei 0 weiter bei um0a Sensor
		brne  um0b								; bei 1 weiter bei um0b RolladenNr
	
		ldi	ZH,High(TextSensor*2)		; Text Sensor ausgeben
		ldi	ZL,Low(TextSensor*2)
		rjmp	um0c								; weiter bei um0c
		
um0b:	ldi	ZH,High(TextRolladen*2)		; Text Rolladen ausgeben
		ldi	ZL,Low(TextRolladen*2)

um0c:	rcall	lcd_print						; Text-Ausgabe
		
		tst	U_MenuFlag						; U-Menflag = 0?
		brne	um0d								; wenn nicht weiter zu Schaltgruppe
		mov	AH,BH								; Wert von BH:BL nach AH:AL fr Ausgabe
		mov	AL,BL
		rcall	PrintSensorWert				; sonst gespeicherten Wert ausgeben
		rjmp	umEnd
		
um0d:	mov	AL,CL								; Wert zur Ausgabe nach AL
		rcall PrintRolladenNr
		rjmp	umEnd								; zum Ende


; Untermen 2-4: Schaltzeit setzen
um1:	cpi	AH,6 							; Untermens 2 bis 4 identisch
		brge  um2							; Wenn Hauptmen >=4 dann weiter

		mov	AL, U_MenuFlag				; wenn Untermen -1, dann
		cpi	AL,-1							; Schaltzeiten und
		brne	um1b							; Programm einlesen

		ldi   ZL,Low(dSZ1R)				; Adresse fr Schalt1runter
		ldi	ZH,High(dSZ1R)				
		cpi	AH,2							; ist AH = 2 = Schaltzeit1runter
		breq	um1a							; ja, dann weiter
		adiw	ZH:ZL,3						; sonst +3 = Schaltzeit1hoch
		cpi	AH,3							; ist AH = 3 = Schaltzeit1hoch
		breq	um1a							; ja, dann weiter
		adiw	ZH:ZL,3						; sonst +3 = Schaltzeit2runter
		cpi	AH,4							; ist AH = 4 = Schaltzeit2runter
		breq 	um1a							; ja, dann weiter
		adiw	ZH:ZL,3						; sonst +3 = Schaltzeit2hoch
		
um1a: ld		BL,Z+         				; Stunde einlesen
		ld		BH,Z+         				; Minute einlesen
		ld		CL,Z          				; Gruppe einlesen

um1b:	inc	AL						     	; wenn Untermen>-1, dann Untermen erhhen
		mov	U_MenuFlag,AL				; ist U-Menu bereits 2
		cpi	AL,3              		; wenn nicht zu "Schaltzeiten setzen"
		brne	um1c
		clr	U_MenuFlag					; sonst Untermen=0

um1c: rcall	Schaltzeit
		rjmp  umEnd
	
;Untermen 6-7: Gruppen definieren
um2:	cpi 	AH,8						
		brge 	um4
		rcall	PrintGruppe
		mov	AL,U_MenuFlag
		cpi	AL,-1                	; Ist Untermen-Flag gesetzt?
		brne	um2a							; wenn ja weiter, sonst
		rcall LCD_CursorOn				; Cursor anschalten
		clr	U_MenuFlag					; U-MenFlag auf 0 setzen
		ser	BH							
		ldi	BL,1							; 1. Zeichen = 1
		rjmp	um2c  						; Ende
	
um2a: ldi	AL,0x43						; 2. Zeile, 3. Spalte
		add	AL,U_MenuFlag				; Cursorposition = U-MenFlag + 0x43
		rcall	LCD_Locate
	
		tst	BH								; BH=Flag Nr. an/aus
		breq	um2b							; wenn 1, dann weiter zur "-"Anzeige
		ldi	AL,0x31						; zur Cursorposition hex31 addieren => Ascii-Zeichen
	   add	AL,U_MenuFlag				; Cursorposition = U-MenFlag
   	rcall LCD_Data						; "1-9" ausgeben
		mov	BL,U_MenuFlag				; Flag zum Speichern im DSEG
		inc	BL								; vorbereiten
		rjmp	um2c							; zum Ende

um2b: ldi	AL,'-'						; sonst "-" nach AL
		rcall	LCD_Data       			; und Zeichen ausgeben
		clr	BL								; Flag zum Speichern im DSEG
	
um2c: ldi	AL,0x43						; nach Ausgabe eines Zeichens wird Cursor weitergesetzt
		add	AL,U_MenuFlag				; daher wieder 1 zurck
		rcall	lcd_locate
	
		ldi	ZH,high(dGr1)           ; Adresse von Gruppe1 holen
		ldi	ZL,low(dGr1)
		cpi	AH,6							; Gruppe1 richtig ?
		breq	um2d							; wenn ja weiter
	   adiw	ZH:ZL,9						; sonst Gruppe2 Adresse=Adresse+9

um2d: add	ZL,U_MenuFlag				; Position = Anfang + MenuFlag
	   st		Z,BL							; und Wert schreiben (gesetzt=Nr, gelscht=0)
		com	BH								; Flag 'setzen/lschen' umdrehen
		ldi 	AL,0x43						; 2. Zeile 2. Spalte hinzuaddieren
		add	AL,U_MenuFlag				; im Flag steht die Cursorposition von 1-9
		rcall	lcd_Locate					; Cursor setzen
		rjmp	umEnd

; Untermen 8: Datum und Uhrzeit setzen
um4:  cpi 	AH,9
		brge	um5
		inc	U_MenuFlag
		mov	AL,U_MenuFlag
		cpi	AL,6							; Ist Max erreicht
		brlt	um4a						 	; wenn nein, weiter
   	clr	U_MenuFlag					; wenn ja, Flag = 0
um4a: rcall	DefDateTime
	   rjmp	umEnd

; Untermen 9: Programme an/aus
um5:	cpi	AH,10
		brge	umEnd
		inc	U_MenuFlag
		mov	AL,U_MenuFlag
		cpi	AL,6							; Ist Max erreicht
		brlo	um5a						 	; wenn nein, weiter
   	clr	U_MenuFlag		
um5a: rcall DefProgAnAus
	   rjmp	umEnd

umEnd:
		pop	AH
		pop	AL
		ret


;*************************************************************************************
; Untermenu 2-5: Schaltzeiten einlesen und anzeigen
; Eingabe: BL=Stunde, BH=Minute, CL=Rolladen					
;*************************************************************************************

Schaltzeit:
  		ldi	AL,0x40						; Cursor auf Anfang setzen
      rcall lcd_locate
  		ldi	ZL,Low(MenuSZ*2)  		; Adresse des Mens laden
 	  	ldi   ZH,High(MenuSZ*2)			; 1. Men = Stunde
		
	   mov	AL, U_Menuflag				; Stunde oder Minute
		tst	AL								; Ist AL = 0 =  Stunde
		brne	sz0							; wenn nicht, weiter
		rcall	lcd_print					; Text "Stunde","Minute" oder "Rolladen" ausgeben
		ldi	AL,0x4A                 ; Cursor schon mal
		rcall	lcd_locate				   ; fr die Ausgabe setzen
		mov	AL,BL							; BL=Stunde nach AL
		rcall LCD_Dec                 ; und ausgeben
		rjmp	szEnd

sz0:	adiw	ZH:ZL,16                ; Text "Minute" auswhlen
		cpi	AL,2
		breq  sz1
		rcall	lcd_print					; Text "Stunde","Minute", "Rolladen" ausgeben
		ldi	AL,0x4A                 ; Cursor schon mal
		rcall	lcd_locate				   ; fr die Ausgabe setzen
		mov	AL, BH			         ; Minute einlesen
		rcall LCD_Dec		            ; und ausgeben
		rjmp	szEnd				
		
sz1:	adiw	ZH:ZL,16                ; Text "Rolladen" auswhlen
		rcall	lcd_print					;
		ldi	AL,0x4A                 ; Cursor schon mal
		rcall	lcd_locate				   ; fr die Ausgabe setzen
		mov	AL, CL						; Rolladen ausgeben
		rcall	PrintRolladenNr

szEnd:
		ret

		
;*************************************************************************************
; Untermenu 8: Datum und Uhrzeit programmieren
;*************************************************************************************
DefDateTime:
		push	AL

  		ldi	ZL,Low(MenuUhr*2)  		; Adresse des Mens laden
 	  	ldi   ZH,High(MenuUhr*2)
	  	
		mov	AL,U_Menuflag
dd1:	tst	AL								; Ist AL=0 dann zu s3 und Ausgabe
		breq	dd3

dd2: 	adiw	ZH:ZL,16						; sonst pro Zhlwert Lnge Mentext addieren
		dec	AL								;
	   brne	dd2								

dd3:	ldi	AL,0x40
		rcall	lcd_locate
		rcall	lcd_print					; Mentext-Ausgabe 2. Zeile 1. Stelle
		
		ldi 	AL, 0x4C						; 2. Zeile 11. Stelle
		rcall LCD_Locate
		mov	AL, U_MenuFlag

		tst 	AL
		brne 	dd4
		mov 	AL, Year
		rjmp	dd9

dd4:	cpi	AL,1
		brne  dd5
		mov 	AL, Month
		rjmp	dd9

dd5:	cpi	AL,2
		brne 	dd6
		mov 	AL, Day
		rjmp	dd9

dd6:	cpi	AL,3							;
		brne	dd7                     ; AL=3 = Wochentag
		rcall	PrintWoTag              ; Wochentag (Zahl von 0-6) auslesen
		rjmp	dd10

dd7:	cpi	AL,4
		brne 	dd8
		mov 	AL, Hour
		rjmp	dd9

dd8:	cpi	AL,5
		brne 	dd9
		mov 	AL, Minute

dd9:	rcall LCD_Dec

dd10:	pop	AL
		ret

;*************************************************************************************
; Untermenu 9: Programme an/aus
;*************************************************************************************
DefProgAnAus:
		ldi	ZH, High(MenuProgr*2)
		ldi	ZL, Low(MenuProgr*2)
		ldi	AH, 1							; Entsprechung im OnOffFlag

dp1:	mov	AL, U_MenuFlag				
		tst	AL                      ; Untermen = 0?
		breq  DefPrintAnAus           ; dann sofort weiter

dp1a:	adiw	ZH:ZL,16                ; Adresse Untermen inkrementieren
		lsl	AH								; Flag nach links	
		dec	AL								; Zhler dekrementieren
		brne	dp1a

DefPrintAnAus:
		ldi	AL, 0x40                ; Cursor 2. Zeile 1. Stelle
		rcall	lCD_Locate
		rcall	LCD_Print               ; Text ausgeben
		ldi 	AL, 0x4F						; Cursor 2. Zeile 11. Stelle
		rcall LCD_Locate		
		mov	AL,OnOffFlag            ; OnOffFlag nach AL
		and	AH,AL                   ; AH mit AL vergleichen
		tst	AH                      ; sofern AH=0, Flag nicht gesetzt->aus
		breq	dp3
		ldi	AL,'a'
		rcall LCD_Data
		ldi	AL,'n'
		rcall LCD_Data
		ldi	AL,' '
		rcall LCD_Data		
		rjmp  dpend

dp3:	ldi	AL,'a'
		rcall LCD_Data
		ldi	AL,'u'
		rcall LCD_Data
		ldi	AL,'s'
		rcall LCD_Data		
dpend:
		ret				


;*************************************************************************************
TasteRunter:
;*************************************************************************************

		push	AL
		push	AH

		mov	AH, H_MenuFlag
		mov	AL, U_MenuFlag
		cpi	AH,-1							; wenn AH=-1 dann Rolladen runter fahren
		brne	tl0
      rcall Rolladen_runter
      rjmp 	tlEnd

;Blttern im Hauptmen
tl0:	cpi	AL,-1	                  ; Ist Untermen >=0
      brne	tl1                     ; wenn nicht +-Hauptmen

		dec	AH								; Hauptmen = Hauptmen-1
		cpi	AH,-1							; ist es schon = -1
		brne	tl0a							; wenn nein weiter
      ldi	AH,9							; sonst auf 9 setzen
tl0a:	mov	H_menuFlag,AH				; Ausgabe Men
		rcall Print_H_Menu
		rjmp	tlEnd

;ADC_Schaltgruppe auswhlen
tl1:	cpi	AH,2						   ; Men 0+1 = Sensor
		brge	tl2                     ; wenn grer weiter
		mov	Al, U_MenuFlag			
		tst	Al								; ist AL=0
		brne	tl1b	                  ; wenn nicht Schaltgruppe bearbeiten

		ldi	AL,10							; AL mit 10 fttern	
		sub	BL,AL							; zu BH:BL addieren
		sbc	BH,Null
		tst	BL
		brne	tl1a
		tst	BH
		brne	tl1a          				; wenn nicht weiter
		ldi	BL,low(1000)				; sonst BH:BL=0
		ldi	BH,high(1000)

tl1a:	mov	AL,BL
		mov	AH,BH
		rcall	PrintSensorwert
		rjmp	tlend		

tl1b:	                        		; Schaltgruppe steht in CL
		dec	CL								; Schaltgruppe=Schaltgruppe-1
		tst	CL			         		; ist es bereits 1. Men
		brge	tl1c							; nein, dann weiter zur Ausgabe
		ldi	CL,11							; sonst Men=12
tl1c:	mov	AL,CL							; CL zur Ausgabe nach AL
		rcall	PrintRolladenNr
      rjmp	tlEnd

; Schaltzeiten
tl2:  cpi	AH,6							; wenn Hauptmen
      brge	tl3
		rcall	tl_Schaltzeit				; <4 dann zu Schaltzeit
		rjmp	tlEnd

; Gruppendefinition		
tl3:	cpi	AH,8							; wenn H-Men>5 dann weiter zu Uhr stellen
		brge	tl4
		mov	BL,U_MenuFlag
		dec	BL								; in BL steht die aktuelle Cursorposition
		cpi	BL,0
		brge  tl3a
		ldi	BL,8
tl3a:	mov	U_menuFlag,BL
		ldi	AL,0x43
		add	AL,BL
		rcall lcd_locate
	 	clr	BH
		rjmp	tlEnd

; Datum/Uhr stellen
tl4:	cpi	AH,9
		brge	tl5
		rcall	tl_DateTime					
      rjmp	tlEnd

; Programme an/aus
tl5:  cpi	AH,9
		brne	tlEnd

		ldi	AH,1
		mov	AL, U_MenuFlag				; Ist Untermen
		tst	AL                      ; = 0
		breq  tl5b                    ; dann sofort weiter

tl5a:	lsl	AH								; Flag nach links	
		dec	AL								; Zhler dekrementieren
		brne	tl5a

tl5b:	eor	OnOffFlag,AH            ; Bit in OnOffFlag umdrehen
		rcall DefPrintAnAus

tlEnd:
		pop	AH
		pop	AL
		ret

;*************************************************************************************
; Rolladen runter fahren
; Relais 1 = ON/Off  Relais2 = hoch/runter
; Flag 0-7 = Rolladen 1-8 / Flag 8 = Rolladen 9
; Flag 9 = Alle / Flag 10/11=Gruppen
;*************************************************************************************

Rolladen_runter:
		push	ZH
		push	ZL
		ldi	AL,0x51						; Pfeil hoch oder runter
		rcall	lcd_locate              ; wieder lschen
		ldi	AL,' '
		rcall	lcd_data

		sbrs	UpDnFlag,0					; wenn Bit0 von UpDnFlag gesetzt ist, fahren gerade
		rjmp	rr_0                    ; Rollden hoch, daher
		rcall	Rolladenaus					; alle Rollden stop
		rjmp	rrEnda						; beenden

rr_0:	mov	AL, RollFlag         	; Rolladen-Nr nach AL
		cpi	AL,8							; Ist Rolladen-Flag > 8 dann weiter
		brge	rr1                 		;

		; zuerst prfen ob Rolladen in Betrieb ist.
		; Wenn ja, Rolladen nicht um- sondern ganz ausschalten

		inc	AL								; AL +1 -> bei Rolladen 1 ist AL=1
		ldi	ZH, High(Bitmuster*2)	; Bitmuster nach Z
		ldi	ZL, Low(Bitmuster*2)

rr0:	lpm	AH,Z+           			; Bitmuster lesen
		dec	AL								; Zhler dekrementieren
		brne	rr0							; wenn nicht 0 dann zurck
		
		in		BL,PortC						; Port einlesen
		tst	BL       					; ist ein Rolladen in Betrieb?
		breq	rr0a							; wenn nein(BL=0), weiter

		mov	AL,AH                   ; Bitmuster nach AH, AH wird spter noch gebraucht
		and	AL,BL							; RolladenBit mit ausgelesenem Port vergleichen
		tst	AL								; ist AH =0, dann war Rolladen nicht in Betrieb
		breq	rr0a							; weiter bei Rolladen-Runter

		com	AL								; sonst Bitmuster umdrehen, gew. Rolladen-Bit ist jetzt 0, alle anderen 1
		and 	BL,AL          			; Rolladen ausschalten, andere unbercksichtigt lassen
		out	PortC,BL						; Neue Bitmaske ausgeben
		out	PortD,BL						; auch Port-D fr Rolladenhoch ausschalten									
		rjmp	rrEnda						; Ende
				
rr0a:	or		BL,AH							; Rolladen-Bit in BL setzen
		out	PortC,BL						; Relais 1 und Strom einschalten
      rjmp	rrEnd

rr1:											; Rolladen 9	 Flag =8
		cpi	AL,9							; PinA6=an/aus PinA7 = 0=runter/1=hoch
		brge	rr2
		
		sbis	PortA,PinA6					; berspringen wenn Bit nicht gesetzt
		rjmp	rr1a
		cbi	PortA,PinA6					; Strom ausschalten
		cbi	PortA,PInA7
		rjmp	rrEnda						; Ende

rr1a:	sbi	PortA,PinA6					; PINA6 setzen -> Strom an
		rjmp	rrEnd

rr2:  in		BL,PortC						; Port C einlesen
      tst	BL								; sind Rollden in Betrieb?
      brne	rr2a                  	; wenn ja, ausschalten
      sbis	PortA,PinA6             ; wenn Rolladen 9 nicht gesetzt
		rjmp	rr2b		               ; weiter bei Rolladen-runter
rr2a:	out	PortC,Null              ; Strom aus
		out 	PortD,Null              ; Richtungsrelais aus
		cbi	PortA,PINA6             ; Strom aus
		cbi	PortA,PinA7					; Richtungsrelais aus
		clr   Ausschaltzeit				; Ausschaltzeit=0
		rjmp	rrEnda						; Ende

rr2b:                               ; weiter, wenn keine Rollden in Betrieb
		cpi	AL,10							; Flag=9 = alle, 10 u. 11 weiter bei Gruppen
      brge	rr4                   	;

		out	PortD,Null					; Richtungsrelais ausschalten
		cbi	PortA,PINA7		

		ldi	AH,25
   	mov	TimeFlag,AH					; Timeflag setzen 25*4ms = 100 ms
rr3: 	tst	TimeFlag						; TimeFlag wird in Timerinterrupt runter gezhlt
		brne	rr3			
		
		ser	BL								; alle Bits setzen = Strom ein
		out	PortC,BL				
		sbi	PORTA,PINA6		
		rjmp	rrEnd

rr4: ; Rolladen Gruppe 1-2
		clr	BL
		cpi	AL,11
		brge  rr4a
		ldi	ZH,high(dGr1)
		ldi	ZL,Low(dGr1)
		rjmp	rr4b

rr4a:	ldi	ZH,high(dGr2)
		ldi	ZL,Low(dGr2)

rr4b:	ld		AL,Z+							; Nr 1 aus Gruppe
		tst	AL                      ; Byte=0
		breq	rr4c	                  ; wenn ja weiter
		ori	BL,1							; Bit0 setzen = Strom ein

rr4c:	ld		AL,Z+							; Nr 2 aus Gruppe
		tst	AL
		breq  rr4d
		ori	BL,2							; Bit1 setzen = Strom ein	

rr4d:	ld		AL,Z+							; Nr 3 aus Gruppe
		tst	AL
		breq  rr4e
		ori	BL,4							; Bit2 setzen = Strom ein			

rr4e:	ld		AL,Z+							; Nr 4 aus Gruppe
		tst	AL
		breq  rr4f
		ori	BL,8							; Bit3 setzen = Strom ein	

rr4f:	ld		AL,Z+							; Nr 5 aus Gruppe
		tst	AL
		breq  rr4g
		ori	BL,16							; Bit4 setzen = Strom ein			

rr4g:	ld		AL,Z+							; Nr 6 aus Gruppe
		tst	AL
		breq  rr4h
		ori	BL,32							; Bit5 setzen = Strom ein	

rr4h:	ld		AL,Z+							; Nr 7 aus Gruppe
		tst	AL
		breq  rr4i
		ori	BL,64							; Bit6 setzen = Strom ein	

rr4i:	ld		AL,Z+							; Nr 8 aus Gruppe
		tst	AL
		breq  rr4k
		ori	BL,128						; Bit7 setzen = Strom ein				

rr4k:	out	PortC,BL						; Rollden 1-8 starten
		
		ld		AL,Z							; Nr 9 aus Gruppe
		tst	AL
		breq  rrEnd
      sbi	PORTA,PINA6					; Rolladen 9 starten

rrEnd:
		ldi	AL,0x51						; Zeichen "Pfeil runter" anzeigen
		rcall	lcd_locate
		ldi	AL,0
		rcall	lcd_data
		lds	Ausschaltzeit,dAusZ
		ldi	Al,2
		mov	UpDnFlag,AL        		; UpDownFlag Bit1 setzen
		
rrEnda:		
		pop	ZL
		pop	ZH
		ret

;-------------------------------------------------------------------------------------
tl_Schaltzeit:

		ldi	AL,0x4A						; schon mal LCD-Ausgabe
		rcall lcd_locate              ; auf 2. Zeile 11. Stelle festlegen
		
		mov	AL,U_menuFlag				; Stunde oder Minute
		tst	AL								; Ist UnterMenu = 0 = Stunde ?
		brne  tlsb                   	; wenn nein dann zu s1
                                    ; in BL steht Schaltstunde
		dec 	BL 							; BL=BL-1
		cpi 	BL,-1							; ist BL bereits -1
		brne  tlsa
		ldi	BL,23							; dann BL = 23
tlsa:	mov	AL,BL							; Zur Ausgabe BL nach AL
		rjmp	tlSend

tlsb: 	                           ; in BH steht Schaltminute
		cpi	AL,1							; Ist UnterMen= Minute
		brne	tlSd							; wenn nicht weiter
		dec	BH								; Minute=Minute-1
		cpi	BH,-1							; ist Minute=-1?
		brne  tlsc			            ; nein dann weiter
		ldi	BH, 59						; sonst Minute =59
tlsc: mov	AL,BH							; zur Ausgabe BH nach AL
      rjmp	tlsend
											
tlSd:											; in CL steht Schaltgruppe
		dec	CL								; Gruppe=Gruppe-1
		cpi	CL,-1							; Ist Gruppe = -1
		brne	tlSe							; Nein dann weiter
		ldi	CL,11							; sonst Gruppe=11
tlSe:	mov	AL,CL							; zur Ausgbae CL nach AL
		rcall	PrintRolladenNr		
		ret

tlSend:
		rcall	LCD_Dec
		ret

;-------------------------------------------------------------------------------------

tl_DateTime:
		ldi 	AL, 0x4C							; 2. Zeile 11. Stelle
		rcall LCD_Locate
		mov   AL, U_MenuFlag
		
		cpi 	AL,1								; wenn Untermen = 1 dann
		brge 	tld1                       ; weiter zu Monat
		dec	year
		brne	tld0
		ldi	year,99
tld0:	mov	AL, year							; nach AL wegen LCD-Ausgabe bertragen
		rjmp	tldEnd

tld1:	cpi	AL,2
		brge 	tld3								; weiter zu Tag
		dec	month								; Monat=Monat-1
		brne 	tld2
		ldi	month,12							
tld2:	mov	AL, Month
		rjmp	tldEnd

tld3:	cpi	AL,3		
		brge 	tld5     						; weiter zu Wochentag
		dec 	day
		brne	tld4
      ldi	day,31
tld4: mov	AL,day
		rjmp	tldEnd

tld5: cpi	AL,4
		brge	tld7
		lds	BL,dWoTag
		dec	BL
		tst	BL
		brge	tld6
		ldi	BL,6
tld6: sts	dWoTag,BL                  ; Wochentag im DSeg speichern
		rcall PrintWoTag					
		ret

tld7:	cpi	AL,5								
		brge 	tld9                       ; weiter zu Minute
		dec 	hour
		brge	tld8
		ldi	hour,23
tld8: mov	AL, hour
		rjmp  tldEnd

tld9:	cpi	AL,5
		brne 	tldEnd
		dec 	Minute
		brge  tld10
		ldi	Minute,59
tld10:mov	AL, Minute

tldEnd:
		rcall LCD_Dec
		ret

				
;*************************************************************************************	
TasteHoch:
;*************************************************************************************

		push	AL
		push	AH

		mov	AH, H_MenuFlag				; H-MenuFlag nach AH
		mov	AL, U_MenuFlag
		cpi	AH,-1							; wenn AH=-1 dann Rolladen hoch fahren
	 	brne	tr0							; wenn Flag >=0 weiter zu Mens	
      rcall Rolladen_hoch				; sonst Rolladen hoch
      rjmp 	trEnd                   ; zum Ende

; im Hauptmen blttern		
tr0:	cpi	AL,-1                   ; Hauptmen ist>=0, ist auch Untermen >=0?
		brne	tr1							; wenn nicht zu Untermens
		inc	AH								; Hauptmen = Hauptmen + 1
		cpi	AH,10							; ist es schon = 10
		brlo	tr0a							; wenn nicht weiter
      clr	AH								; sonst auf 0 setzen

tr0a: mov	H_MenuFlag,AH				; Ausgabe Men
		rcall Print_H_Menu
    	rjmp	trEnd


; ADC-Sensor-Vergleichswert
tr1:	cpi	AH,2							; Hauptmen = 0 od. 1?
		brge	tr2  							; wenn nicht, weiter
      tst	AL								; Untermen =0 ?
		brne	tr1c							; wenn nicht, weiter

		ldi	AL,10							; AL mit 10 fttern
		add	BL,AL							; zu BH:BL addieren
		adc	BH,Null
		cpi	BL, low(1000)				; ist BH:BL = 1000
		brne	tr1b
		cpi	BH, high(1000)
		brne	tr1b        				; wenn nicht weiter
		clr	BL								; sonst BH:BL=0
		clr	BH
tr1b:		
		mov	AL,BL
		mov	AH,BH
		rcall	PrintSensorWert
		rjmp	trend		
tr1c:											; ADC-Gruppenauswahl
		inc 	CL								; in CL steht Schaltgruppe
		cpi	CL, 12						; ist es bereits 12. Men
		brne	tr1d							; nein, dann weiter zur Ausgabe
		clr	CL								; sonst 0
		
tr1d:	mov	AL,CL							; Zur Ausgabe CL anch AL
		rcall	PrintRolladenNr
      rjmp	trend


;Schaltzeiten
tr2:  cpi	AH,6							; ist Hauptmen-Flag>=6
      brge	tr3	                  ; weiter zu Gruppendefinition
		rcall	tr_Schaltzeit				; wenn <4 dann zu Schaltzeit
		rjmp	trend

; Gruppendefinition		
tr3:  cpi	AH,8							; wenn H-Men>8 dann weiter zu Uhr stellen
		brge	tr4
		mov	BL, U_MenuFlag
		inc	BL 							; in BL steht die aktuelle Cursorposition
		cpi	BL,9
		brne  tr3a
		ldi	BL,0
tr3a:	mov	U_MenuFlag, BL
		ldi	AL,0x43
		add	AL,BL
		rcall lcd_locate
		clr	BH
		rjmp	trend

; Datum/Uhr stellen
tr4:	cpi	AH,9
		brge	tr5
		rcall	tr_DateTime					
      rjmp	trend


; Programme an/aus
tr5:  cpi	AH,9
		brne	trEnd
		ldi	AH,1
		mov	AL, U_MenuFlag				; Ist Untermen
		tst	AL                      ; = 0
		breq  tr5b                    ; dann sofort weiter
tr5a:
		lsl	AH								; Flag nach links	
		dec	AL								; Zhler dekrementieren
		brne	tr5a

tr5b:	eor	OnOffFlag,AH            ; Bit in OnOffFlag umdrehen
		rcall DefPrintAnAus

trEnd:
		pop	AH
		pop	AL
		ret

;*************************************************************************************
;Rolladen hoch fahren
;Relais 1 = ON/Off  Relais2 = hoch/runter
;Flag 0-7 = Rolladen 1-8 / Flag 8 = Rolladen 9
;Flag 9 = Alle / Flag 10+11 = Gruppen
;*************************************************************************************

Rolladen_hoch:	; Relais On/Off an Pin1 ausschalten
		
		
		push	ZH
		push	ZL		

		ldi	AL,0x51						; Pfeil hoch oder runter
		rcall	lcd_locate              ; lschen
		ldi	AL,' '
		rcall	lcd_data
     				
		sbrs	UpDnFlag,1					; wenn Bit1 von UpDnFlag gesetzt ist, fahren gerade
		rjmp	rh0a                    ; Rollden runter, daher
		rcall	Rolladenaus					; alle Rollden stop
		rjmp	rhEnda						; beenden

rh0a: mov	AL,RollFlag         		; Rolladen-Nr nach AL
		cpi	AL,8							; Ist Rolladen-Nr >=8 dann weiter bei tr10
		brge	rh3	                 	;
		
		; zuerst prfen ob Rolladen in Betrieb ist.
		; Wenn ja, Rolladen nicht um- sondern ganz ausschalten
		inc	AL								; Flag um 1 erhhen bei Roll. 1 ist jetzt Bit1 gesetzt
		ldi	ZH, High(Bitmuster*2)   ; Bitmuster nach Z
		ldi	ZL, LOw(Bitmuster*2)

rh0:	lpm	AH,Z+							; Bitmuster lesen
		dec	AL                      ; Zhler dekrementieren
		brne	rh0	                  ; wenn nicht 0, dann zurck

		in		BL,PortC						; Port einlesen
		tst	BL       					; ist ein Rolladen in Betrieb?
		breq	rh1							; wenn nein(BL=0), weiter

		mov 	AL,AH							; Bitmuster nach AL
		and  	AL,BL							; ausmaskieren
		tst	AL								; wenn AL = 0 dann war BL auch 0
		breq	rh1							; d. h. Rolladen mit Flag Rollflag ist nicht in Betrieb, daher weiter
  		
  		com	AL								; umdrehen
  		and 	BL,AL  		            ; in BL ausmaskieren
  		out 	PortC,BL                ; am Port (Strom)ausgeben
  		out 	PortD,BL						; Port Richtung
  		rjmp	rhEnda						; Ende
		
rh1:	;Relais hoch/runter einschalten
		
		or		BL,AH                   ; Bit in BL "anschalten"
		out	PortD,BL						; und auf PortD (Richtungsrelais)ausgeben

      ;100 ms warten
		ldi	AL,25
   	mov	TimeFlag,AL					; Timeflag setzen 25*4ms = 100 ms
rh2:	tst	TimeFlag						; TimeFlag wird in Timerinterrupt runter gezhlt
		brne	rh2

		;Relais 1 anschalten
		out	PortC,BL						; Strom einschalten
      rjmp	rhEnd

rh3:	cpi	AL,9                    ; Rollflag =8 / Rolladen Nr. 9
		brge	rh6

		sbis	PortA,PinA6					; wenn Strom an, dann ausschalten
		rjmp	rh4                   	; wenn nicht, weiter
		cbi	PortA,PINA6					; erstmal stromlos schalten
		cbi	PortA,PINA7					; Richtungsrelais ausschalten
		rjmp	rhEnda						; Ende

rh4:	sbi	PortA,PinA7					; Richtung einschalten

		ldi	AL,25							
   	mov	TimeFlag,AL					; Timeflag setzen 25*4ms = 100 ms
rh5:	tst	timeFlag						
		brne	rh5
		
		sbi	PortA,PINA6					; Strom einschalten
		rjmp	rhEnd							; beenden


rh6:; alle Rollden und Gruppen

      in		BL,PortC						; Port C einlesen
      tst	BL								; sind Rollden in Betrieb? d.h. BL>0
      brne	rh6a                   	; wenn ja weiter bei 13a
   	sbis	PortA,PinA6					; ist PinA6 gesetzt=Rolladen9, wenn nein
   	rjmp	rh6b							; dann weiter bei rr2b
   	
rh6a:	clr	BL								
		out	PortC,BL						; Strom aus
		out	PortD,BL						; Richtung aus
		cbi	PortA,PinA6				   ; Strom aus
		cbi	PortA,PinA7					; Richtung aus
		clr   Ausschaltzeit
		rjmp	rhEnda						; Ende
		
rh6b: ;hier weiter, wenn keine Rollden in Betrieb
		
      cpi	AL,10							; Rollflag 9 =alle, 10/11= Gruppen
      brge	rh7                   	; weiter, wenn Gruppen

		ser	BL								; BL=FF
		out	PortD,BL                ; Richtungsrelais 1-8 ein
		sbi	PortA,PinA7             ; Richtungsrelais 9 ein
      										
		ldi	AL,25
   	mov	TimeFlag,AL					; Timeflag setzen 25*4ms = 100 ms
rh6c:	tst	TimeFlag						; TimeFlag wird in Timerinterrupt runter gezhlt
		brne	rh6c
		
		out	PortC,BL				      ; Strom ein 1-8
		sbi	PORTA,PINA6             ; Strom ein 9
		rjmp	rhEnd


rh7: ; Rolladen Gruppe 1-2				; evtl laufende Rollden wurden bereits oben
		clr	BL								; ausgeschaltet
		cpi	AL,11
		brge	rh7a
		ldi	ZH,high(dGr1)
		ldi	ZL,Low(dGr1)
		rjmp	rh7b

rh7a:	ldi	ZH,high(dGr2)
		ldi	ZL,Low(dGr2)

rh7b:	ld		AL,Z+							; Nr 1 aus Gruppe
		tst	AL                      ; wenn Bit=0, Rolladen nicht gesetzt
		breq	rh7c                  	; sonst Bit in Byte setzen
		ori	BL,1							; Bit0 setzen

rh7c:	ld		AL,Z+							; Nr 2 aus Gruppe
		tst	AL
		breq 	rh7d
		ori	BL,2							; Bit1 setzen

rh7d:	ld		AL,Z+							; Nr 3 aus Gruppe
		tst	AL
		breq  rh7e
		ori	BL,4							; Bit2 setzen = Strom ein

rh7e:	ld		AL,Z+							; Nr 4 aus Gruppe
		tst	AL
		breq  rh7f
		ori	BL,8							; Bit3 setzen = Strom ein	

rh7f:	ld		AL,Z+							; Nr 5 aus Gruppe
		tst	AL
		breq  rh7g
		ori	BL,16							; Bit4 setzen = Strom ein	

rh7g:	ld		AL,Z+							; Nr 6 aus Gruppe
		tst	AL
		breq  rh7h
		ori	BL,32							; Bit5 setzen = Strom ein	

rh7h:	ld		AL,Z+							; Nr 7 aus Gruppe
		tst	AL
		breq  rh7i
		ori	BL,64							; Bit6 setzen = Strom ein	

rh7i:	ld		AL,Z+							; Nr 8 aus Gruppe
		tst	AL
		breq  rh7k
		ori	BL,128						; Bit7 setzen = Strom ein	

rh7k:	out	PortD,BL						; Richtungsrelais ein

		ldi	AL,25
		mov	Timeflag,AL					; 100 ms Warten
rh7l:	tst	Timeflag
		brne	rh7l
		
		out	PortC,BL						; die Rollden 1-8 starten, Stromrelais ein

		ld		AL,Z							; Nr 9 aus Gruppe
		tst	AL
		breq  rhEnd
      sbi	PORTA,PINA7					; Richtungsrelais 9 setzen

		ldi	AH,25
		mov	Timeflag,AH					; 100 ms Warten
rh7m:	tst	Timeflag
		brne	rh7m

		sbi	PORTA,PINA6					; Stromrelais 9 setzen	

rhEnd:ldi	AL,0x51						; Zeichen "Pfeil hoch" anzeigen
		rcall	lcd_locate
		ldi	AL,1
		rcall	lcd_data
		lds	AusschaltZeit,dAusZ
		ldi	AL,1
		mov	UpDnFlag,AL					; UpDownFlag Bit0 setzen

rhEnda:
		pop	ZL
		pop	ZH
		ret
	
;-------------------------------------------------------------------------------------
tr_Schaltzeit:

		ldi	AL,0x4A					; schon mal LCD-Ausgabe
		rcall lcd_locate           ; festlegen

		mov	AL,U_menuFlag			; Stunde oder Minute
		tst	AL
		brne  trS1
		
		inc	BL							; BL = Schaltstunde
		cpi	BL,24						; ist BL schon 24
		brne  trS0						; nein, dann weiter
		clr	BL							; BL=0

trS0:	mov	AL,BL						; Zur Ausgabe BL nach AL
		rjmp  trSE

trS1:	cpi	AL,1						; Unter-Men=1 = Minute
		brne  trS3                 ; wenn nicht zur Schaltgruppe
		inc	BH							; BH=Schaltminute, BH = BH + 1
		cpi	BH,60						; Minute schon 60
		brne  trS2						; nein, dann weiter
		clr	BH							; sonst Minute=0

trS2:	mov	AL,BH						; Zur Ausgabe nach AL
		rjmp	trse

trS3:	inc	CL							; CL=Schaltgruppe
		cpi	CL,12						; Ist Gruppe bereits 12?
		brne	trS4						; nein dann weiter
		clr 	CL							; Gruppe=0
		
trS4: mov	AL,CL						; Zur Ausgabe Gruppe nach AL
		rcall	PrintRolladenNr		; Ausgabe
		ret

trSE:	rcall	LCD_Dec
      ret

;-------------------------------------------------------------------------------------
tr_DateTime:
	
		ldi 	AL, 0x4C							; 1. Zeile 11. Stelle
		rcall LCD_Locate
		mov   AL, U_MenuFlag

		cpi 	AL,1								; Jahr keine Obergrenze
		brge 	tru4
		inc	year
		mov 	AL,Year
		rjmp	tru8

tru4:	cpi	AL,2								; Monat ?
		brge 	tru5								; gehe zu Tag
		inc	Month
		cpi	Month,13
		brne	tru4b
      ldi	Month,1
tru4b:mov	AL, Month		
   	rjmp	tru8

tru5:	cpi	AL,3								; Tag?
		brge 	tru5c							   ; gehe zu Wochentag
		inc 	Day
		cpi	Day,32
		brne	tru5b
		ldi	Day,1
tru5b:mov	AL,Day
		rjmp	tru8

tru5c:cpi	AL,4
		brge	tru6                       ; gehe zu Stunde
      lds	BL,dWoTag						; Wochentag(Zahl) aus DSEG lesen
      inc	BL									; erhhen
      cpi	BL,7                       ; ist es schon 7
      brne	tru5d                      ; wenn nein weiter
      clr	BL		                     ; sonst auf 0 setzen
tru5d:sts	dWoTag,BL						; Wochentag speichern
		rcall PrintWoTag						; ausgeben
		ret	

tru6:	cpi	AL,5								; Stunde ?
		brge 	tru7                       ; gehe zu Minuten
		inc	Hour
		cpi	Hour,24
		brne	tru6b
		clr	Hour
tru6b:mov	AL, Hour
		rjmp	tru8

tru7:	cpi	AL,5
		brne 	tru8
		inc	Minute
		cpi	Minute,60
		brne	tru7b
		clr	Minute
tru7b:mov	AL, Minute

tru8:	rcall LCD_Dec
		ret
		

;*************************************************************************************
; Taste4 = Aufruf Hauptmen, zurck zum Hauptmen, wenn Untermen aktiv
; wenn AH=-1 d.h kein Hauptmen, Hauptme auf 0 setzen und 1. HM anzeigen
; wenn AH<>-1 d. h. Exit je nach Untermen Ergebnisse speichern und beenden
;*************************************************************************************

TasteRechts:
		push	AL
		push	AH
	
		clr	Ausschaltzeit				; Falls gerade ein Rolladen luft, ausschalten
   	rcall LCD_CursorOff				; Cursor ggf. ausschalten
		
		mov	AH,H_MenuFlag				;
		cpi	AH,-1                   ; Hauptmen-Flag gesetzt ?
		brne	te2                     ; wenn ja, weiter bei te2
		ldi	AH,9							; "Progamme an/aus" = 1. Men
		mov	H_MenuFlag,AH
		rcall	Print_H_Menu				; Hauptmen anzeigen
		pop	AH                      ; und zurck
		pop	AL
		ret
		
te2:	mov	AL,U_MenuFlag				;
		cpi	AL,-1							; wenn Untermen <> -1 dann
		brne  te3					      ; ab te3 evtl. neue Werte abspeichern
		rjmp	teEnd                   ; sonst Men abbrechen und Datum/Uhrzeit zeigen


;ADC speichern
te3:	cpi	AH,2		
      brge	te4                     ; wenn Hauptmenu>= 2 weiter bei Schaltzeiten
		
		ldi   ZL,Low(dSensR)		      ; Adresse im DSEG fr SensorRunter
		ldi	ZH,High(dSensR)
		sbrc	AH,0							; nchsten Befehl berspringen, wenn AH=0
		adiw	ZH:ZL,3                 ; sonst Adresse im DSEG fr Sensorhoch

		st		Z+,BH							; Werte ins DSEG Schreiben
		st		Z+,BL
		st		Z, CL							; Schaltgruppe
		rjmp	teEnd

; Schaltzeiten speichern	
te4:	cpi	AH,6							; Ist MenuFlag = 4 od.5
		brge	te6							; wenn nicht, dann zur Gruppendefinition
	
      ldi   ZL,Low(dSZ1R)		      ; Adresse Schalt1Runter
		ldi	ZH,High(dSZ1R)				;
		cpi	AH,2							; wenn Hauptmen = 2 = Schalt1runter
		breq  te5							; zum Schreiben
		adiw	ZH:ZL,3						; sonst Adresse SchaltZeit1hoch
		cpi	AH,3							; wenn Men = 3 dann schreiben
		breq	te5							; zum Schreiben
		adiw	ZH:ZL,3						; Adresse SchaltZeit2Runter
		cpi	AH,4                    ; wenn Men = 4 dann Schreiben
		breq  te5							; zum Schreiben
		adiw	ZH:ZL,3						; Men5 SchaltZeit2Hoch
te5:	st		Z+,BL	;Schaltstunde     ; Werte im DSEG speichern
		st		Z+,BH	;Schaltminute
		st		Z, CL	;Schaltgruppe
      rjmp	teEnd

; Gruppendefinition
;te6:	cpi	AH,8							; MenuFlag 6 od. 7 ?
;		brge	te7							; wenn nicht, weiter
;		rjmp	teEnd							; zum Ende

;Datum/Uhrzeit	setzen	
te6:	cpi	AH,8							; MenuFlag =8 ?
		brne	te8							; wenn nicht, weiter
		clr	Second						; Sekundenzhler bei 0 beginnen lassen
		sei                     		; Interrupts wieder zulassen
		rjmp	teEnd

;OnOffFlag sichern
te8:	cpi	AH,9							; MenuFlag=9 ?
		brne	teEnd							; wenn nicht weiter
		sts	dOnOff,OnOffFlag			; OnOffFlag speichern
	
teEnd:
		ldi	AL,-1
		mov	U_MenuFlag,AL     		; MenFlags lschen
		mov	H_MenuFlag,AL
		rcall	LCD_Clear
		rcall PrintDateTime
		rcall	PrintRolladen
		rcall	PrintSensor
		rcall PrintStatus
		
      pop	AH
		pop	AL
		ret

;*************************************************************************************		
; Alle laufenden Rollden ausschalten
;*************************************************************************************
Rolladenaus:
		clr	BL								;
		out   PortC,BL
		out	PortD,BL
		cbi	PortA,PinA6
		cbi	PortA,PinA7
		clr	UpDnFlag
		ldi	AL,0x51						; Pfeil hoch oder runter
		rcall	lcd_locate              ; wieder lschen
		ldi	AL,' '
		rcall	lcd_data
		ret				

;*************************************************************************************		
; Text "Sensor" und aktuellen Wert in 3. Display-Zeile 11. Stelle ausgeben
;*************************************************************************************
PrintSensor:
		ldi	AL,0x14						; 3. Zeile 1. Stelle
		rcall	LCD_Locate
 	 	ldi   ZL,Low(TextSensor*2)  	; Text "Sensor:" ausgeben
	  	ldi	ZH, High(TextSensor*2)
		rcall	lcd_print
PrintSensVal:
		ldi	AL,0x20						; 3. Zeile 10. Stelle
		rcall	LCD_Locate	
	   mov	AL,ADCLow					; RolladenFlag nach AL
	   mov	AH,ADCHigh
	   rcall	bin2ascii
		push	AL                      ; Wert steht in BH:BL,AH:AL
		push	AH                      ; max 1023
		push	BL
		mov	AL,BH							; zuerst BH
		rcall	lcd_data		            ; ausgeben
		pop	AL          				; BL holen
		rcall	lcd_data                ; ausgeben
		pop	AL                      ; dito AH und AL
		rcall	lcd_data
		pop	AL
		rcall	lcd_data	
		ret	

; Wert fr Einstellung ausgeben, Wert in AH:AL, Register BL,BH,CL sichern
PrintSensorWert:
	   push	AL
		ldi	AL,0x4C                 ; 2. Zeile 11. Spalte
		rcall	LCD_Locate
		pop	AL
		push	CL
		push	BH
		push	BL
		rcall	bin2ascii
		push	AL                      ; Wert steht in BH:BL,AH:AL
		push	AH                      ; max 1023
		push	BL
		mov	AL,BH							; zuerst BH
		rcall	lcd_data		            ; ausgeben
		pop	AL          				; BL holen
		rcall	lcd_data                ; ausgeben
		pop	AL                      ; dito AH und AL
		rcall	lcd_data
		pop	AL
		rcall	lcd_data	
		pop	BL
		pop	BH
		pop	CL
		ret	


;*************************************************************************************		
; Text "Rolladen" und/oder Nr. in 2. Display-Zeile 11. Stelle ausgeben
;*************************************************************************************
PrintRolladen:
		ldi	AL,0x40						; 2. Zeile 1. Stelle
		rcall	LCD_Locate
 	 	ldi   ZL,Low(TextRolladen*2)  ; Text "Rolladen:" ausgeben
	  	ldi	ZH, High(TextRolladen*2)
		rcall	lcd_print
	   mov	AL,RollFlag					; RolladenFlag nach AL

PrintRolladenNr:							; erwartet Nr. in AL
		ldi	ZL,low(MenuRL*2)
		ldi	ZH,high(MenuRL*2)
		tst	AL								; ist es bereits 0 also Roll.-Nr. 1
		breq	pre							; dann zur Ausgabe

pr1:  dec	AL								; ansonsten runterzhlen
		adiw	ZH:ZL,6						; dabei Menadresse hochzhlen
		tst	AL
		brne	pr1							

pre:	ldi	AL,0x4C						; 2. Zeile 10. Stelle
		rcall	LCD_Locate	
		rcall	lcd_print
		ret


;*************************************************************************************
;Datum und Uhrzeit in 1. Displayzeile ausgeben
;*************************************************************************************

PrintDateTime:
		rcall LCD_Home				; Cursor auf 1. Stelle
		rcall	PrintWoTag
		ldi	AL,','
		rcall	lcd_data				; Komma ausgeben		
		ldi	AL,' '				; Leer nach AL
		rcall	lcd_data				; ausgeben
		mov	AL, Day				; Tag nach AL
		rcall	LCD_Dec				; ausgeben
		ldi	AL,'.'				; Punkt nach AL
		rcall	lcd_data				; ausgeben
		mov	AL, Month			; Monat
		rcall	LCD_Dec
		ldi	AL,'.'				; Punkt
		rcall	lcd_data
		mov	AL, Year				; Jahr
		rcall	LCD_Dec
		ldi	AL, ' '				; Lehrzeichen
		rcall lcd_data
		mov	AL, Hour				; Stunde
		rcall	LCD_Dec
		ldi	AL, ':'				; :
		rcall lcd_data
		mov	AL, Minute			; Minute
		rcall	LCD_Dec
		ldi	AL, ' '				; Lehrzeichen
		rcall lcd_data
		ldi	AL, ' '				; Lehrzeichen
		rcall lcd_data
	   ret

;*************************************************************************************
; Ausgabe Wochentag, Cursor muss vorher gesetzt sein
;*************************************************************************************
PrintWoTag:
		push	ZH
		push	ZL
		push	AL
		lds	AL,dWoTag
		ldi	ZH,high(Wochentag*2)
		ldi	ZL,low(Wochentag*2)
		tst	AL							; ist AL = 0 (Sonntag)
		breq	pwt1	 	            ; wenn Ja weiter
pwt0:	adiw	ZH:ZL,4					; sonst 4 dazuzhlen
		dec	AL          	      ; Schleifenzhler -1
		brne	pwt0	          	   ; wenn nicht 0 zurck
pwt1:	rcall LCD_Print				; in ZH:ZL steht Wochentag					
		pop	AL
		pop	ZL
		pop	ZH
		ret	


;***************************************************************************
;*
;* LCD-Routinen  4-Bit
;*                                             	
;* PIN_LCD_Enable	= PB5									
;* PIN_LCD_RS		= PB4									
;* PIN_LCD_D4		= PB0									
;* PIN_LCD_D5		= PB1									
;* PIN_LCD_D6 		= PB2									
;* PIN_LCD_D7		= PB3									
;*																
;****************************************************************************

;sendet Datenbyte an das LCD
lcd_data:
		push	AL
	   swap  AL				   	   				; Obere 4 bits (Nibble) nach unten
  		andi  AL, 0b00001111         				; 0bere Hlfte auf Null setzen
		sbr	AL, (1<<PIN_LCD_RS)					; PIN_LCD_RS						
		out	LCD_PORT,AL
		rcall lcd_enable             				; Enable-Routine aufrufen
   	pop	AL											; 2. Nibble	
		andi  AL, 0b00001111         				; obere Hlfte auf Null setzen
		sbr	AL, (1<<PIN_LCD_RS)					; PIN_LCD_RS
		out	LCD_Port,AL
		rcall lcd_enable             				; Enable-Routine aufrufen
		rcall wait50us              				; warte
		ret                            			; zurck zum Hauptprogramm

;sendet einen Befehl an das LCD
lcd_command:                            		; wie lcd_data, nur ohne RS zu setzen
		cbi	LCD_Port, PIN_LCD_RS
		push	AL
	   swap	AL
		andi	AL, 0b00001111
		out	LCD_Port,AL
		rcall	lcd_enable
	   pop	AL
		andi	AL, 0b00001111
		out	LCD_Port,AL
		rcall	lcd_enable
	   rcall	wait50us
	   ret


 ;erzeugt den Enable-Puls
lcd_enable:
 		sbi 	LCD_Port, PIN_LCD_Enable    		; Enable high
	   nop                          				; 3 Taktzyklen warten
		nop
		nop
		cbi	LCD_Port, PIN_LCD_Enable     		; Enable wieder low
		ret                          				; und wieder zurck


 ;Initialisierung
lcd_init:
	  	ldi	AL,10
li0:	rcall	wait5ms
		dec	AL
		brne	li0

		ldi	AL, 0b00000011         		
   	out	LCD_Port, AL              		
		rcall	lcd_enable             				; 1. muss 3mal hintereinander gesendet
   	rcall	wait5ms									;    werden zur Initialisierung
	   rcall	lcd_enable             				; 2.
   	rcall	wait5ms
	   rcall	lcd_enable             				; und 3!
   	rcall	wait5ms
	
		ldi	AL,	0b00000010						; 4bit Modus einstellen
		out	LCD_Port,AL
		rcall	lcd_enable
		rcall	Wait5ms	

		ldi	AL, 0b00101000         				; 4bit-Modus, 2 Zeilen
   	rcall	lcd_command								; Befehl senden
		ldi	AL, 0b00001100         				; LCD on
   	rcall	lcd_command								; Befehl senden
		ldi	AL, 0b00000100         				; Entry mode
   	rcall	lcd_command
	   ret

;Display lschen
lcd_Clear:
   	ldi	AL, 0b00000001         				; Display lschen
	   rcall	lcd_command								; Cursor 1. Zeile/1.Spalte
   	rcall	Wait5ms
	   ret

; Strich-Cursor anschalten
lcd_CursorOn:
		ldi	AL,0b00001110							; 1+Display ein+Cursor ein+Blinken aus
		rcall	lcd_command
   	rcall	Wait50us
	   ret

; Strich-Cursor ausschalten
lcd_CursorOff:
		ldi	AL,0b00001100							; 1+Display ein+Cursor aus+Blinken aus
		rcall	lcd_command
   	rcall	Wait50us
	   ret

;Setzt Cursor-Position auf Anfang
lcd_Home:
		ldi	AL, 0b00000010         				; Cursor an Anfang
	  	rcall	lcd_command
  		rcall	wait5ms
	  	ret	

; Cursor setzen
lcd_Locate:
		sbr	AL, 0b10000000         				; Bit 7 addieren
		rcall	lcd_command
		rcall	wait5ms
	   ret

; Text-Ausgabe
lcd_Print:
		lpm										; Adresse von Z laden
		tst	R0									; ist Zeichen = 0
		breq	lcd_pend							; -> Ende
		mov	AL,R0								; sonst Zeichen nach AL
		rcall	lcd_data							; Ausgabe
		adiw	ZL,1              			; Zeiger in ZL = ZL + 1(word)
		rjmp	lcd_print				
lcd_pend:
		ret

;Dezimal-Ausgabe mit fhrender Null fr LCD
LCD_Dec:
		mov	AH,AL
		ldi	AL,0x2F								; Ascii "0" -1
ld1:
		inc	AL
		subi	AH,10
		brcc	ld1
		rcall	lcd_data
		mov	AL,AH
		subi	AL, 0xC6 							; 256-Ascii"0"-10
		rcall	lcd_data
		ret

;*************************************************************************
;Binr nach Ascii-Umwandlung fr LCD
;input:  AH, AL / output: CL,BH,BL,AH,AL = 5 digits (ASCII)
;*************************************************************************

bin2ascii:
		ldi  CL,-1+'0'
	
bcd1: inc  CL
      subi AL, low(10000)
      sbci AH, high(10000)
      brcc bcd1
      ldi  BH, 10 + '0'

bcd2: dec  BH
      subi AL, low(-1000)
      sbci AH, high(-1000)
      brcs bcd2
      ldi  BL, -1 + '0'

bcd3: inc  BL
      subi AL, low(100)
      sbci AH, high(100)
      brcc bcd3
      ldi  AH, 10 + '0'

bcd4: dec  AH
      subi AL, -10
      brcs bcd4
      subi AL, -'0'

		ret

;***************************************************************************
; Warteschleifen
;****************************************************************************

wait50us:
   	push  AL
		ldi	AL, (xtal * 50 / 3) / 1000000
wait50us_:
		dec	AL
		brne	wait50us_
		pop	AL
		ret
	
wait5ms:
		push	AL
		push	AH
		ldi	AL, (xtal * 5 / 607) / 1000
wgloop0:
		ldi	AH,0xC9
wgloop1:
		dec	AH
		brne	wgloop1
		dec	AL
		brne	wgloop0
   	pop	AH
		pop	AL
		ret

;***************************************************************************************
; im DSEG gespeicherte Schaltzeiten mit aktueller Zeit vergleichen und ggf.
; Rollden bewegen. Da der Sensor morgens und abends den gleichen Wert
; messen kann, SensorHoch nur vor 12 Uhr, SensorRunter nach 12 Uhr zulassen.
;***************************************************************************************
SensorAlarm: ; Programm Sensor runter

		tst	Ausschaltzeit				; nur wenn Ausschaltzeit=0
		breq	sa0							; weiter
		ret									; sonst zurck, da Rollden bereits laufen
		
sa0:	push	ZH
		push	ZL
		push 	Rollflag
	   sbrs	OnOffFlag,0					; ist Bit1  gesetzt?
	   rjmp	sa1                     ; wenn nicht, weiter
		
		cpi	Hour,12						; runter soll's nur am Abend > 12 Uhr gehen
		brlo	sa1							;
		ldi	ZH,High(dSensR)			; Wert aus DSEG fr SensorRunter
      ldi   ZL,Low(dSensR)				
		ld		AH,Z+						
		ld		AL,Z+
		cp		ADCLow,AL
		cpc	ADCHigh,AH					; ist akt. Wert < Schaltwert
		brcc	sa1							; wenn nein weiter
		subi	AL,10							; Hysterese von 10
		sbc	AH,Null						;
		cp		ADCLow,AL
		cpc	ADCHigh,AH					; ist akt. Wert > Schaltwert+Hysterese von 10
		brcs	sa1							; wenn nein weiter		
		ld		Rollflag,Z					; Schaltgruppe holen
		rcall	Rolladen_runter         ; und Rolladen runter fahren
		rjmp	sa2

sa1:	sbrs	OnOffFlag,1					; Ist Sensor hoch angeschaltet
		rjmp	sa2							; wenn nicht, Ende
		cpi	Hour,12  					; hoch soll's nur
		brge	sa2							; am Morgen < 12 Uhr gehen
		ldi	ZH,High(dSensH)			; Wert aus DSEG fr SensorHoch
      ldi   ZL,Low(dSensH)				
      ld		AH,Z+
      ld		AL,Z+
		cp		AL,ADClow					; ist Wert > akt. Wert
		cpc	AH,ADChigh					;
	   brcc	sa2							; wenn nein Ende
		ldi	BL,10							; Hysterese von 10
		add	AL,BL
		adc	AH,Null						;
		cp		AL,ADClow					; ist Wert+Hysterese > akt. Wert
		cpc	AH,ADChigh					;				
		brcs	sa2							; wenn nein Ende
		ld		Rollflag,Z          		; wenn High- und LowByte gleich, Schaltgruppe holen
		rcall	Rolladen_hoch	         ; und Rolladen hoch fahren
sa2:
		pop	Rollflag
		pop	ZL
		pop	ZH
		ret
		
TimerAlarm:	; Programm Timer1 runter
		push	ZH
		push	ZL
		push	Rollflag
		sbrs	OnOffFlag,2             ; wenn Bit2 nicht gesetzt, weiter zu Prog1hoch
		rjmp	at1b
		ldi	ZH,High(dSZ1R)				; Stunde aus DSEG fr Schalt1runter
      ldi   ZL,Low(dSZ1R)				; Rolladen runter fahren?
		ld	   AH,Z+
      cp		Hour, AH						; ist Schaltstunde = akt. Stunde				
		brne	at1b							; wenn nein weiter bei loop		
		ld		AH,Z+
		cp		Minute, AH					; ist Schaltminute = akt. Minute
		brne	at1b							; wenn nein, weiter bei loop
      ld		Rollflag,Z		      	; Schaltgruppe holen
		rcall	Rolladen_runter         ; und Rolladen runter fahren
      rjmp	AlarmEnde		

at1b:	; Programm Timer1 hoch
      sbrs	OnOffFlag,3             ; wenn Bit2 nicht gesetzt, weiter zu Timer2
		rjmp	AlarmTim2
		ldi	ZH,High(dSZ1H)				; Stunde aus DSEG fr Schalt1runter
      ldi   ZL,Low(dSZ1H)				
		ld	   AH,Z+
		cp		Hour, AH						; ist Schaltstunde = akt. Stunde				
		brne	AlarmTim2					; wenn nein weiter bei loop		
      ld		AH,Z+
		cp		Minute, AH					; ist Schaltminute = akt. Minute
		brne	AlarmTim2					; wenn nein, weiter bei loop
		ld		Rollflag,Z              ; Schaltgruppe holen
		rcall	Rolladen_hoch	         ; und Rolladen hoch fahren
		rjmp	AlarmEnde		

AlarmTim2:	; Programm Timer2 runter
		sbrs	OnOffFlag,4
		rjmp	at2b
		
		ldi	ZH,High(dSZ2R)				; Stunde aus DSEG fr Schalt2runter
      ldi   ZL,Low(dSZ2R)				; Rolladen runter fahren?
		ld	   AH,Z+
		cp		Hour, AH						; ist Schaltstunde = akt. Stunde				
		brne	at2b							; wenn nein weiter
		ld		AH,Z+
		cp		Minute, AH					; ist Schaltminute = akt. Minute
		brne	at2b							; wenn nein, weiter
		ld		Rollflag,Z              ; Schaltgruppe noch holen
		rcall	Rolladen_runter         ; und Rolladen runter fahren
      rjmp	AlarmEnde		

at2b:	; Programm Timer2 hoch
		sbrs	OnOffFlag,5
		rjmp	AlarmEnde
		ldi	ZH,High(dSZ2H)				; Stunde aus DSEG fr Schalt2hoch
      ldi   ZL,Low(dSZ2H)				
		ld	   AH,Z+
		cp		Hour, AH						; ist Schaltstunde = akt. Stunde				
		brne	AlarmEnde					; wenn nein Ende
		ld		AH,Z+
		cp		Minute, AH					; ist Schaltminute = akt. Minute
		brne	AlarmEnde					; wenn nein, Ende
		ld		Rollflag,Z              ; Schaltgruppe noch holen				
		rcall	Rolladen_hoch	         ; und Rolladen hoch fahren

AlarmEnde:
		pop	Rollflag
		pop	ZL
		pop	ZH
		ret

;***************************************************************************************
;ADC starten: Addition von 255 Werten, Rckgabe Mittelwert in ADCHigh und ADCLow
;***************************************************************************************
GetADC:
		push	AL
		push	AH
		push	BL
		push	BH
		clr	AL
		clr	AH
		clr	BL
		ser	BH		

ad0:  sbi	ADCSR,ADSC
ad1:  sbic	ADCSR,ADSC
		rjmp	ad1
		in 	ADCLow, ADCL
		in 	ADCHigh,ADCH
		add 	AL, ADCLow
		adc   AH, ADCHigh
		adc	BL, Null
		dec	BH
		brne	ad0
		mov	ADCLow,AH
		mov	ADCHigh,BL
		pop	BH
		pop	BL
		pop	AH
		pop	AL
		ret

;***************************************************************************************
; Ausgabe ADC-Wert auf LCD (0 = 0 Volt bis 1023 = 5 Volt)
;***************************************************************************************
PrintADC:
		push	AL
		push	AH
		push	BL
		push	BH
		push	CL
		push	ZL
		push	ZH
				
		ldi	AL,0x40							; 2. Zeile 1. Spalte
		rcall	lcd_locate
		ldi	ZH,High(Sensortext*2)		; Text ausgeben
		ldi	ZL,Low(sensortext*2)
		rcall	lcd_print
		
		ldi	AL,0x43                    ; 2. Zeile 4 Spalte
		rcall	lcd_Locate
		mov	AL,ADCLow						; Wert ausgeben
		mov	AH,ADCHigh
		rcall bin2Ascii 						
		push	AL                         ; Wert steht in BH:BL,AH:AL
		push	AH                         ; max 1023
		push	BL
		mov	AL,BH								; zuerst BH
		rcall	lcd_data		               ; ausgeben
		pop	AL          					; BL holen
		rcall	lcd_data                   ; ausgeben
		pop	AL                         ; dito AH und AL
		rcall	lcd_data
		pop	AL
		rcall	lcd_data	
		pop	ZH
		pop	ZL
		pop	CL
		pop	BH
		pop	BL
		pop	AH
		pop	AL
		ret	


;***************************************************************************************
; im Eeprom gespeicherte Daten ins DSEG kopieren
;***************************************************************************************
ReadEeprom:
		ldi	ZH,High(SensR)				; Anfang Eeprom
      ldi   ZL,Low(SensR)				
		ldi	YH,High(dSensR)			; Anfang DSeg
		ldi	YL,Low(dSensR)

		ldi	AL,38							; AL = Zhler 38 mal
re0:	out	EEARH,ZH						; Eeprom lesen
		out	EEARL,ZL
		sbi	EECR,EERE
		in 	AH,EEDR      				;
		st		Y+,AH							; ins DSeg bertragen
		adiw	ZH:ZL,1
		dec	AL
		brne	re0	
		mov	OnOffFlag,AH            ; Letzter Wert ist OnOffFlag
		ret

SaveEeprom:
      in    isreg, sreg
		ldi   YL,Low(dSensR)		      ; Adresse im Eeprom fr Sensorhoch auswhlen
		ldi	YH,High(dSensR)			; weiter bei ts1
		ldi	ZL,Low(SensR)
		ldi   ZH,High(SensR)

		ldi	AL,38							; Zhler Anzahl Stellen
      cli                        	; Interrupts ausschalten
se0:  sbic	EECR,EEWE					; warten bis Eeprom bereit
		rjmp	se0	
		out	EEARH,ZH						; Eeprom schreiben
		out	EEARL,ZL
		ld		AH,Y+							; dSEG auslesen
		out 	EEDR, AH						; ins Eeprom schreiben
		sbi   EECR, EEMWE
      sbi   EECR, EEWE
		adiw	ZH:ZL,1
		dec	AL
		brne  se0
		out	sreg,isreg
		sei
		ret


;***************************************************************************************
; Status von Sensor, Timer1 und Timer2 anzeigen
;***************************************************************************************

PrintStatus:
		ldi	ZL,Low(Status * 2)			; Text Statuszeile nach Z
		ldi	ZH,high(Status * 2)
		ldi	AL,0x54							; 4. Zeile 1. Spalte
		rcall	lcd_locate
		rcall	lcd_print					   ; ausgeben
		
		ldi	AL,0x56							; Cursor fr Ausgabe
   	rcall	lcd_locate

		sbrs	OnOffFlag,0						; ist Bit 0 (Sensor runter) gesetzt?
		rjmp  ps0                        ; wenn nicht zu ps0
	   ldi	AL,'r'; 0						; sonst "Pfeilrunter" ausgeben
   	rcall lcd_data
	   rjmp	ps1a
ps0:  ldi	AL,'-'							
	   rcall lcd_data

ps1a: sbrs	OnOffFlag,1						; ist Bit 1 (Sensor hoch) gesetzt?
		rjmp  ps1b								; wenn nicht zu ps1a
   	ldi	AL,'h'; 1                       ; sonst "Pfeilhoch" ausgeben
	   rcall lcd_data
   	rjmp	ps2
ps1b: ldi	AL,'-'
	   rcall lcd_data

ps2: 	 											; Flag fr Pragramm1:runter
	   ldi	AL,0x5D  						; Cursor fr Ausgabe Prog1-Flag
   	rcall	lcd_locate

	   sbrs	OnOffFlag,2						; Bit2 = Programm1 runter
   	rjmp	ps2a								; wenn Bit nicht gesetzt weiter bei ps2a
		ldi	AL,0								; "Pfeilrunter"
   	rcall lcd_data
	   rjmp	ps3
ps2a: ldi	AL,'-'
	   rcall lcd_data	

ps3: 	sbrs	OnOffFlag,3						; Flag fr Pragramm1:hoch
		rjmp  ps3a
   	ldi	AL,1								; "Pfeilhoch"
	   rcall lcd_data
   	rjmp	ps4
ps3a: ldi	AL,'-'
	   rcall lcd_data

ps4:  ldi	AL,0x64
	   rcall	lcd_locate

		sbrs	OnOffFlag,4						; Flag fr Programm2 runter
		rjmp	ps4a
   	ldi	AL,0								; "Pfeilrunter"
	   rcall lcd_data
   	rjmp	ps5
ps4a: ldi	AL,'-'
	   rcall lcd_data	

ps5:	sbrs	OnOffFlag,5
		rjmp	ps5a
   	ldi	AL,1								; "Pfeilhoch"
	   rcall lcd_data
   	rjmp	psend
ps5a: ldi	AL,'-'
	   rcall lcd_data	

psend:ret

;***************************************************************************************
; Gruppe anzeigen bei Definition
;***************************************************************************************
PrintGruppe:
		ldi	ZH, high(dGr1)					; Adresse Gruppe1 nach Z
		ldi	ZL, Low(dGr1)
		cpi	AH,7								; Ist Hauptmen=7 = Gruppe2
		brne	pg0								; wenn nicht weiter
		adiw	ZH:ZL,9							; sonst Adresse Gruppe2
pg0:  ldi	AL,0x43							; Cursor auf 2. Zeile 3. Stelle
	   rcall	lcd_locate						
		ldi	CL,9
		ldi	BL,0x30
pg1:  ld		AL,Z+                      ; Zeichen an Z nach AL 1-9
		tst	AL									; ist Zeichen=0
		breq	pg2								; wenn ja, weiter bei pg2
		add	AL,BL								; sonst AL in Ascci wandeln
		rjmp	pg3								; und weiter bei pg3
pg2:  ldi	AL,0x2D							; wenn Zeichen=0 dann Zeichen = "-"
pg3:  rcall lcd_data							; Zeichen ausgeben
		dec	CL									; Cursor +1
		brne	pg1
		ret

;***************************************************************************************
; Benutzerdefinierte Zeichen einlesen
;***************************************************************************************
LCD_LoadChars:
		ldi	ZL, low(LCD_Chars*2)
		ldi	ZH, high(LCD_Chars*2)
   	clr	BL

llc0:	clr	AH

llc1:	ldi	AL,0b01000000
		add	AL,BL
		add	AL,AH
	   rcall	LCD_command
   	lpm	AL,Z+
	   rcall	LCD_data
   	ldi	AL,0b01001000
	   add	AL,BL
   	add	AL,AH
	   rcall	LCD_Command
   	lpm	AL,Z+
	   rcall LCD_Data
   	inc	AH
	   cpi	AH,8
   	brne	llc1

		subi	BL,-0x10
		lpm	AL,Z
		cpi	AL,0xFF
		brne	llc0
		ret


;***************************************************************************************
; Texte
;***************************************************************************************

Text_H_Menu:
	.db "Sensor: runter  ",0,0  ;H-Menu-Flag 0
	.db "Sensor: hoch    ",0,0  ;1
	.db "Timer1: runter  ",0,0  ;2
	.db "Timer1: hoch    ",0,0  ;3
	.db "Timer2: runter  ",0,0  ;4
	.db "Timer2: hoch    ",0,0  ;5
	.db "Def.: Gruppe 1  ",0,0  ;6
	.db "Def.: Gruppe 2  ",0,0  ;7
	.db "Def.: Datum/Uhr ",0,0  ;8
	.db "Programme an/aus",0,0  ;9

MenuRL:
	.db "01  ",0,0
	.db "02  ",0,0
	.db "03  ",0,0
	.db "04  ",0,0
	.db "05  ",0,0
	.db "06  ",0,0
	.db "07  ",0,0
	.db "08  ",0,0
	.db "09  ",0,0
	.db "Alle",0,0
	.db "Gr.1",0,0
	.db "Gr.2",0,0

MenuProgr:	
	.db "Sensor runter:",0,0
	.db "Sensor hoch:  ",0,0
	.db "Timer1 runter:",0,0
	.db "Timer1 hoch:  ",0,0
	.db "Timer2 runter:",0,0
	.db "Timer2 hoch:  ",0,0		

MenuUhr:
	.db "Jahr:          ",0
	.db "Monat:         ",0
	.db "Tag:           ",0
	.db "Wochentag:     ",0	
	.db "Stunde:        ",0
	.db "Minute:        ",0

MenuSZ:
	.db "Stunde:        ",0
	.db "Minute:        ",0
	.db "Rolladen:		  ",0		

TextRolladen:
   .db "Rolladen:",0

TextSensor:
	.db "Sensor  :",0

Gruppe:
	.db "123456789",0

Status:
	.db "S:    T1:    T2:   ",0

Sensortext:
	.db "W:       R:        ",0

Wochentag:
	.db "So",0,0  ;Tag 0
	.db "Mo",0,0
	.db "Di",0,0
	.db "Mi",0,0
	.db "Do",0,0
	.db "Fr",0,0
	.db "Sa",0,0  ;Tag 6

LCD_Chars:
	.db	0b00000, 0b00100             ;            |
	.db	0b00100,	0b01010             ;     |    /   \
	.db	0b00100,	0b10101             ;     |   /  |  \
	.db	0b00100,	0b00100             ;     |      |
	.db	0b00100,	0b00100             ;     |      |
	.db	0b10101, 0b00100             ;  \  | /    |
	.db	0b01010, 0b00100             ;   \  /     |
	.db	0b00100, 0b00000             ;     |
	.db	0xFF,0xFF

BitMuster:
	.db	1,2,4,8,16,32,64,128

.dseg
dSensR: .Byte 3		; ADC0-Wert runter
dSensH: .Byte 3		; ADC0-Wert hoch
dSZ1R:  .Byte 3	   ; Timer1runter: Stunde, Minute, Gruppe
dSZ1H:  .Byte 3		; Timer1hoch:   Stunde, Minute, Gruppe
dSZ2R:  .Byte 3		; Timer2runter: Stunde, Minute, Gruppe
dSZ2H:  .Byte 3		; Timer2hoch:   Stunde, Minute, Gruppe
dGr1:   .Byte 9		; Gruppe1
dGr2:   .Byte 9	   ; Gruppe2
dAusZ:  .Byte 1		; Ausschaltzeit
dOnOff: .Byte 1		; OnOffFlag
dWoTag: .Byte 1		; Wochentag

.eseg
SensR: .db 0,0x64,9				; ADC0-Wert runter, Gruppe
SensH: .db 0,0xC8,9				; ADC0-Wert hoch, Gruppe
SZ1R:  .db 0,0,9  				; Timer1runter: Stunde, Minute, Gruppe
SZ1H:  .db 0,0,9					; Timer1hoch:   Stunde, Minute, Gruppe
SZ2R:  .db 0,0,9					; Timer2runter: Stunde, Minute, Gruppe
SZ2H:  .db 0,0,9					; Timer2hoch:   Stunde, Minute, Gruppe
Gr1:	 .db 1,2,3,4,5,6,0,0,9	; Gruppe1 9Byte
Gr2:	 .db 0,0,0,0,0,0,7,8,0 	; Gruppe2 9Byte
AusZ:	 .db 35						; Ausschaltzeit in Sekunden
OnOff: .db 128						; Bit 0/1 = Sensor runter/hoch = Wert 1/2
										; Bit 2/3 = Timer1 runter/hoch = Wert 4/8
										; Bit 4/5 = Timer 2 runter/hoch= Wert 16/32
										;
										; Bit 7   = Sommer/Winter = Wert 128/0





















